# This script...


import sys
from PyQt5 import QtGui
from PyQt5.QtWidgets import QApplication, QMainWindow, QPushButton, QLabel, QLineEdit, QAction 
from PyQt5.QtCore import QDate, QTime
import serial
import threading

heaterTimer = 0
timer_is_set = 'false'

class Window(QMainWindow):
    def __init__(self):
        super().__init__()
        self.title = "Smart Block Heater (PyQt5)"
        self.top = 100
        self.left = 100
        self.width = 350
        self.height = 350
        self.setWindowIcon(QtGui.QIcon("logo_32pix.png"))
        exitApp = QAction("Quit", self)
        exitApp.triggered.connect(self.closeEvent)
        oneSecondTimer = 0
        self.init_ui()

    def init_ui(self):
        self.setWindowTitle(self.title)
        self.setGeometry(self.top, self.left, self.width, self.height)
        # time 
        self.cTimeLable = QLabel("", self)
        self.cTimeLable.move(40, 20)
        self.cTimeLable.resize(300, 30)
        self.cTimeLable.setFont(QtGui.QFont('SansSerif', 11))
        
        # set button
        self.setButton = QPushButton("Set ready Time", self)
        self.setButton.clicked.connect(self.set_button_clicked)
        self.setButton.move(45, 80)

        # set time
        self.hhLineEdit = QLineEdit(str(QTime.currentTime().hour()+1), self)
        self.hhLineEdit.resize(30, 28)
        self.hhLineEdit.move(170, 80)
        self.timeSpacer = QLabel(":", self)
        self.timeSpacer.move(210, 80)
        self.timeSpacer.resize(10, 28)
        
        self.mmLineEdit = QLineEdit("00", self)
        self.mmLineEdit.resize(30, 28)
        self.mmLineEdit.move(225, 80)
        self.timeSpacer = QLabel(":", self)
        self.timeSpacer.move(265, 80)
        self.timeSpacer.resize(10, 28)
        
        self.ssLineEdit = QLineEdit("00", self)
        self.ssLineEdit.resize(30, 28)
        self.ssLineEdit.move(280, 80)
      
        # cord
        self.imgCord = QLabel(self)
        self.imgCord.setPixmap(QtGui.QPixmap("cord.png"))
        self.imgCord.move(45, 170)
        self.cordButton = QPushButton("Up / Down", self)
        self.cordButton.clicked.connect(self.cord_button_clicked)
        self.cordButton.move(47, 215)

        #Temperature
        self.tmpLable = QLabel("29 ℃", self)
        self.tmpLable.move(201, 170)
        self.tmpLable.resize(140, 50)
        self.tmpLable.setFont(QtGui.QFont('SansSerif', 26))
        
        #rTime
        self.rTimeLable = QLabel("", self)
        self.rTimeLable.move(45, 270)
        self.rTimeLable.resize(300, 50)
        self.rTimeLable.setFont(QtGui.QFont('SansSerif', 14))
        self.rTimeLable.setStyleSheet('color: blue')

        #message
        self.msgLable = QLabel("", self)
        self.msgLable.move(45, 270)
        self.msgLable.resize(300, 50)
        self.msgLable.setFont(QtGui.QFont('SansSerif', 14))
        self.msgLable.setStyleSheet('color: green')
        
        
        self.arduinoData = serial.Serial('COM4', 9600)
        

        self.show()

    def set_button_clicked(self):
        global timer_is_set
        timer_is_set = 'true'
        self.msgLable.setText('')

    def cord_button_clicked(self):
        #self.arduinoData.Write('1')
        pass

    def timeCounter(self):
        self.cTime = QTime.currentTime().toString()
        self.cTimeLable.setText("Current Time: " + self.cTime)
        self.timer.start()

    def timer(self): # each 1 sec
        global timer_is_set
        global heaterTimer
        cTime = QTime.currentTime()
        self.cTimeLable.setText("Current Time:  " + cTime.toString())
        if (timer_is_set == 'true'):
            cHH = QTime.currentTime().hour()
            cMM = QTime.currentTime().minute()
            cSS = QTime.currentTime().second()
            setHH = int(self.hhLineEdit.text())
            setMM = int(self.mmLineEdit.text())
            setSS = int(self.ssLineEdit.text())
            rHH = setHH - cHH
            rMM = setMM - cMM
            rSS = setSS - cSS
            rSeconds = rHH*3600+rMM*60+rSS
            rTime = QTime(0,0,0).addSecs(rSeconds-5)
            
            if (rSeconds >= 5):
                self.rTimeLable.setText(rTime.toString() + " to start")
            #heating time is set to 5 second (just for test)
            elif (heaterTimer < 5): 
                if (heaterTimer == 0):
                    self.arduinoData.write(str.encode('1'))
                    self.rTimeLable.setText('')
                self.msgLable.setText("Heater is ON:  " +
                                      QTime(0,0,0).addSecs(5-heaterTimer).toString())
                heaterTimer = heaterTimer + 1
            
            elif (heaterTimer == 5):
                self.arduinoData.write(str.encode('0'))
                self.msgLable.setText('Heater is OFF')
                timer_is_set = 0
                heaterTimer = 0
        
        self.oneSecondTimer = threading.Timer(1, self.timer).start()

    def closeEvent():
        self.oneSecondTimer.cancel()



if __name__=="__main__":
    app = QApplication(sys.argv)
    window = Window()
    window.timer()


    sys.exit(app.exec())
